/*
  Класс парсера пакета
*/

package ru.vershinin.medical.lelmon;

public class CommandParser {
    final public char availParams[] = {'M', 'D', 'S', 'P', 'T', 'A', 'R', 'V'};
    final public char startOfPacket = '>';
    final public char spaceOfPacket = ':';
    final public char endOfPacket = ';';
    public char parameter;
    public short value;
    public short crc8;
    public short actualCrc8;

    // Возвращает true, если Ch - это символ HEX-числа [0..9, A..F]
    public boolean checkHexDigit(char Ch) {
        return (Ch >= 'A' && Ch <= 'F') || (Ch >= '0' && Ch <= '9');
    }

    // Преобразует один символ HEX-числа [0..9, A..F] в DEC-число [0..15]
    public byte hex2Digit(char Ch) {
        byte Result;

        if (Ch >= 'A' && Ch <= 'F')
            Result = (byte) (Ch - 55);
        else
            Result = (byte) (Ch - 48);
        return Result;
    }

    // Проверяет корректность значения параметра
    public boolean paramIsValid(char param) {
        boolean Result = false;

        for (int i = 0; i < availParams.length; i++) {
            if (param == availParams[i])
            {
                Result = true;
                break;
            }
        }
        return Result;
    }

    // Рассчет CRC8
    public short calcCrc8(byte[] block, int len) {
        int p, i;
        byte Result = (byte)0xFF;

        p = 0;
        while (len-- != 0) {
            Result = (byte)((Result & 0xFF) ^ (block[p++] & 0xFF));

            for(i = 0; i < 8; i++) {
                Result = (byte)(((Result & 0xFF) & 0x80) != 0 ? (Result & 0xFF) << 1 ^ 0x31 : (Result & 0xFF) << 1);
            }
        }

        return (short)(Result & 0xFF);
    }

    // Парсер пакета. Возвращает коды ошибок.
    public int parse(String Src) {

        // Проверка длины пакета
        if (Src.length() != 8) return 1;

        // Проверка символов стартового, разделительного и завершающего
        if (Src.charAt(0) != startOfPacket) return 2;
        if (Src.charAt(4) != spaceOfPacket) return 3;
        if (Src.charAt(7) != endOfPacket) return 4;

        // Проверка корректности символов величины параметра и CRC
        if (checkHexDigit(Src.charAt(2)) == false) return 5;
        if (checkHexDigit(Src.charAt(2 + 1)) == false) return 5;
        if (checkHexDigit(Src.charAt(5)) == false) return 6;
        if (checkHexDigit(Src.charAt(5 + 1)) == false) return 6;

        // Получение значения параметра
        parameter = Src.charAt(1);
        if (paramIsValid(parameter) == false) return 7;

        // Получение значений: величины и CRC
        value = (short) ((hex2Digit(Src.charAt(2)) << 4) + hex2Digit(Src.charAt(2 + 1)));
        crc8 = (short) ((hex2Digit(Src.charAt(5)) << 4) + hex2Digit(Src.charAt(5 + 1)));

        // Расчет и проверка CRC8
        actualCrc8 = calcCrc8(Src.getBytes(), 5);
        if (actualCrc8 != crc8) return 8;

        return 0;
    }
}
