/*
Copyright (C) 2016, Vershinin Ivan
*/

package ru.vershinin.medical.lelmon;

import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.text.Html;
import android.util.Log;
import android.util.TypedValue;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.GridView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class MainActivity extends Activity {

    public static final String TAG = "MAIN_ACTIVITY";

    // Код запроса диалога (намерения) включения Bluetooth
    private static final int REQUEST_ENABLE_BT = 3;

    // Визуальные элементы окна
    LinearLayout layMain;
    private GridView gvSlots;
    private List<Slot> slots;
    private SlotAdapter slotAdapter;
    private TextView tvMode;
    private TextView tvPreset;
    private TextView tvVersion;

    private LelDevice lelDevice = new LelDevice();
    // Имя подключенного устройства
    private String deviceName = null;
    // MAC-адрес Bluetooth модуля ("20:15:02:26:02:22")
    static String deviceMacAddr;
    // Флаг "прибор найден"
    private boolean deviceIsFound = false;
    // Строковый буфер для исходящих сообщений
    private StringBuffer outStringBuffer;
    // Bluetooth адаптер своего телефона
    private BluetoothAdapter btAdapter = null;
    // Служба Bluetooth-соединения
    private BluetoothService btService = null;
    // Хендлер для приема и обработки сообщений
    private Handler btHandler;
    // Прогресс-диалог при обнаружении Bluetooth-устройств
    private ProgressDialog progressDialog;
    // Флаг "вид подключения"
    private boolean secureConnect;
    // StringBuilder для обработки входящих сообщений по каналу Bluetooth
    private StringBuffer sb = new StringBuffer();
    private StringBuffer packet = new StringBuffer();
    int beginOfPacketIndex = 0;
    private CommandParser cmdParser = new CommandParser();
    int bgIndex = 0;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Bluetooth адаптер своего телефона
        btAdapter = BluetoothAdapter.getDefaultAdapter();

        // Если btAdapter = null - Bluetooth не поддерживается, завершить приложение
        if (btAdapter == null) {
            Toast.makeText(this, R.string.bt_not_support, Toast.LENGTH_LONG).show();
            finish();
        }

        // Инициализировать переменные пользовательского интерфейса
        layMain =  (LinearLayout) findViewById(R.id.layMain);
        gvSlots = (GridView) findViewById(R.id.gvSlots);
        slots = new ArrayList<Slot>();
        slots.add(new Slot(0, 0, 0)); // progNum, preTime, PWM
        slots.add(new Slot(-1, -1, -1));
        slots.add(new Slot(-1, -1, -1));
        slots.add(new Slot(-1, -1, -1));
        slots.add(new Slot(-1, -1, -1));
        slotAdapter = new SlotAdapter(this, slots, lelDevice);
        gvSlots.setAdapter(slotAdapter);

        tvMode = (TextView) findViewById(R.id.tvMode);
        tvPreset = (TextView) findViewById(R.id.tvPreset);
        tvVersion = (TextView) findViewById(R.id.tvVersion);

        tvMode.setText("?");
        tvPreset.setText("?");
        tvVersion.setText("?");

        // Зарегистрировать BroadcastReceiver для отслеживания события обнаружения
        IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
        this.registerReceiver(brReceiver, filter);

        // Зарегистрировать BroadcastReceiver для отслеживания событий окончания обнаружения
        filter = new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
        this.registerReceiver(brReceiver, filter);

        // Зарегистрировать BroadcastReceiver для отслеживания событий включения/выключения Bluetooth
        filter = new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED);
        this.registerReceiver(brReceiver, filter);

        // Handler приема информации от службы Bluetooth
        btHandler = new Handler() {
            @Override
            public void handleMessage(android.os.Message msg) {
                int endOfPacketIndex;

                switch (msg.what) {
                    case Constants.MESSAGE_STATE_CHANGE:
                        switch (msg.arg1) {
                            case BluetoothService.STATE_CONNECTED:
                                setStatus(getString(R.string.title_connected_to, deviceName));
                                break;
                            case BluetoothService.STATE_CONNECTING:
                                setStatus(R.string.title_connecting);
                                break;
                            case BluetoothService.STATE_LISTEN:
                            case BluetoothService.STATE_NONE:
                                setStatus(R.string.title_not_connected);
                                break;
                        }
                        break;
                    case Constants.MESSAGE_WRITE:
                        byte[] writeBuf = (byte[]) msg.obj;
                        // Показ переданного по каналу Bluetooth сообщения
                        String writeMessage = new String(writeBuf);
                        break;
                    case Constants.MESSAGE_READ:
                        // Прием и обработка входящих сообщений по каналу Bluetooth
                        byte[] readBuf = (byte[]) msg.obj;
                        String readMessage = new String(readBuf, 0, msg.arg1);
                        Log.i(TAG, "MESSAGE_READ..........................." + readMessage);
                        sb.append(readMessage);                               // формируем строку
                        endOfPacketIndex = sb.indexOf(";"); //  определяем символ конца пакета
                        while (endOfPacketIndex > 0)
                        {
                            prepareCommand(endOfPacketIndex);
                            execCommand();
                            endOfPacketIndex = sb.indexOf(";");
                        }
                        break;
                    case Constants.MESSAGE_DEVICE_NAME:
                        // Запомнить имя устройства
                        deviceName = msg.getData().getString(Constants.DEVICE_NAME);
                        Toast.makeText(MainActivity.this, getString(R.string.msg_connected) + deviceName, Toast.LENGTH_SHORT).show();
                        break;
                    case Constants.MESSAGE_TOAST:
                        Toast.makeText(MainActivity.this, msg.getData().getString(Constants.TOAST), Toast.LENGTH_SHORT).show();
                        break;
                }
            }
        };

        layMain.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.d(TAG, "Change Background..........");
                bgIndex++;
                if (bgIndex > 3)
                    bgIndex = 0;
                switch (bgIndex) {
                    case 1:
                        layMain.setBackgroundResource(R.drawable.bg1);
                        break;
                    case 2:
                        layMain.setBackgroundResource(R.drawable.bg2);
                        break;
                    case 3:
                        layMain.setBackgroundResource(R.drawable.bg3);
                        break;
                    default:
                        layMain.setBackgroundResource(R.drawable.bg0);
                        break;
                }
            }
        });
    }

    @Override
    public void onStart() {
        super.onStart();
        // Если Bluetooth не включен - запросить включение, тогда setupConnection() будет
        // вызван в процессе onActivityResult
        if (!btAdapter.isEnabled()) {
            Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
            // Если Bluetooth включен - настроить Bluetooth-соединения
        } else if (btService == null) {
            setupConnection();
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        // Остановить службу Bluetooth
        if (btService != null) {
            btService.stop();
        }
        // Остановить обнаружение
        if (btAdapter != null) {
            btAdapter.cancelDiscovery();
        }
        // Дерегистрировать BroadcastReceiver
        this.unregisterReceiver(brReceiver);
    }

    @Override
    public void onResume() {
        super.onResume();
        // Начальный текст полей для принятого и передаваемого текста
        // Запуск службы Bluetooth
        if (btService != null) {
            // Только если состояние STATE_NONE - служба не запущена, запустить
            if (btService.getState() == BluetoothService.STATE_NONE) {
                btService.start();
            }
        }
    }

    // Настройка Bluetooth-соединения
    private void setupConnection() {
        Log.d(TAG, "setupConnection()");
        // Создание BluetoothService
        btService = new BluetoothService(this, btHandler);
        // Создание буфера исходящих сообщений
        outStringBuffer = new StringBuffer("");
    }

    // Обновить статус на ActionBar
    private void setStatus(int resId) {
        final ActionBar actionBar = getActionBar();
        if (null == actionBar) {
            return;
        }
        actionBar.setSubtitle(resId);
    }

    // Обновить статус на ActionBar
    private void setStatus(CharSequence subTitle) {
        final ActionBar actionBar = getActionBar();
        if (null == actionBar) {
            return;
        }
        actionBar.setSubtitle(subTitle);
    }

    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case REQUEST_ENABLE_BT:
                // Юзер подтвердил включение Bluetooth
                if (resultCode == Activity.RESULT_OK) {
                    // setup Bluetooth
                    setupConnection();
                } else {
                    // Юзер не разрешил включение Bluetooth - завершение приложения
                    Log.d(TAG, "BT not enabled");
                    Toast.makeText(this, R.string.bt_not_enabled_leaving, Toast.LENGTH_SHORT).show();
                    finish();
                }
        }
    }

    // Установить соединение
    private void connectDevice() {
        BluetoothDevice device = btAdapter.getRemoteDevice(deviceMacAddr);
        // Попытка подключиться
        btService.connect(device, secureConnect);
    }

    // Поиск имени прибора в списке спаренных устройств. Инициализирует deviceMacAddr
    public void findPairedDevice()
    {
        Set<BluetoothDevice> pairedDevices = btAdapter.getBondedDevices();

        deviceIsFound = false;
        if (pairedDevices.size() > 0) // Если список спаренных устройств не пуст
        {
            for (BluetoothDevice device: pairedDevices) // проходимся в цикле по этому списку
            {
                if (Constants.LEL_DEVICE_NAME.equals(device.getName()))
                {
                    deviceMacAddr = device.getAddress();
                    deviceIsFound = true;
                    break;
                }
            }
        }
    }

    // Вызов соединения
    private void connectToDevice()
    {
        // Если Bluetooth сейчас включен - поиск/обнаружение и подключение
        if (btAdapter.isEnabled()) {
            // Поиск имени прибора в списке спаренных устройств
            findPairedDevice();
            if (!deviceIsFound) // Прибора нет в списке спаренных - запуск обнаружения
            {
                progressDialog = ProgressDialog.show(this, this.getString(R.string.msg_find), this.getString(R.string.msg_wait));
                // После обнаружения - подключение в brReceiver.onReceive
                btAdapter.startDiscovery();
            } else // Прибор ЕСТЬ в списке спаренных - подключаемся
            {
                connectDevice();
            }
        }
    }

    // События при обнаружении, окончании процесса обнаружения и включении/выключении Bluetooth
    private final BroadcastReceiver brReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {

            String action = intent.getAction();
            if (BluetoothDevice.ACTION_FOUND.equals(action)) {

                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                if (Constants.LEL_DEVICE_NAME.equals(device.getName()))
                {
                    deviceMacAddr = device.getAddress();
                    deviceIsFound = true;
                    btAdapter.cancelDiscovery();
                    // Система сама запросит пароль
                    connectDevice();
                }
            } else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {
                // По окончании обнаружения скрыть progressDialog
                if (progressDialog != null)
                    progressDialog.dismiss();
            } else if (action.equals(BluetoothAdapter.ACTION_STATE_CHANGED)) {
                // События включения/выключения Bluetooth
                final int state = intent.getIntExtra(BluetoothAdapter.EXTRA_STATE, BluetoothAdapter.ERROR);
                switch (state) {
                    case BluetoothAdapter.STATE_OFF:
                        finish(); // Завершение приложения при выключении Bluetooth
                        break;
                    case BluetoothAdapter.STATE_TURNING_OFF:
                        break;
                    case BluetoothAdapter.STATE_ON:
                        break;
                    case BluetoothAdapter.STATE_TURNING_ON:
                        break;
                }
            }
        }
    };

    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Создать меню по шаблону "menu_main"
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId()) {
            case R.id.secure_connect_scan: {
                // Пункт меню - "Соединить"
                secureConnect = true;
                connectToDevice();
                return true;
            }
            case R.id.show_app_about: {
                // Пункт меню - "О комплексе "Лель"
                AlertDialog.Builder builder = new AlertDialog.Builder(MainActivity.this);
                builder.setTitle(R.string.app_about_title)
                        .setMessage(Html.fromHtml(getString(R.string.app_about)))
                        .setIcon(R.drawable.ic_launcher)
                        .setCancelable(true)
                        .setNegativeButton("ОК",
                                new DialogInterface.OnClickListener() {
                                    public void onClick(DialogInterface dialog, int id) {
                                        dialog.cancel();
                                    }
                                });
                AlertDialog alert = builder.create();
                alert.show();
                TextView txtView = (TextView) alert.findViewById(android.R.id.message);
                txtView.setTextSize(TypedValue.COMPLEX_UNIT_PX, 22);
                return true;
            }
        }
        return false;
    }

    private void prepareCommand(int endOfPacketIndex)
    {
        int beginOfPacketIndex;

        if (endOfPacketIndex > 7)
            beginOfPacketIndex = endOfPacketIndex - 7;
        else
            beginOfPacketIndex = 0;
        packet.setLength(0);
        packet.append(sb.substring(beginOfPacketIndex, endOfPacketIndex + 1));
        sb.delete(0, endOfPacketIndex + 1);
    }

    private void execCommand()
    {
        Log.i(TAG, "COMMAND..........................." + packet);
        if (cmdParser.parse(packet.toString()) == 0) {
            switch (cmdParser.parameter) {
                case 'M': {
                    if (cmdParser.value == (Constants.END_MODE + 1)) { // Процедура прервана
                        lelDevice.mode = Constants.END_MODE;
                        tvMode.setText(lelDevice.modeToString(Constants.END_MODE + 1));
                    } else {
                        lelDevice.mode = cmdParser.value;
                        tvMode.setText(lelDevice.modeToString(lelDevice.mode));
                    }
                    break;
                }
                case 'D': {
                    lelDevice.preset = cmdParser.value;
                    if (lelDevice.preset < 8)
                        //tvPreset.setText(android.text.Html.fromHtml(getString(R.string.lbl_preset, lelDevice.preset + 1)));
                        tvPreset.setText(String.valueOf(lelDevice.preset + 1));
                    else
                        tvPreset.setText("?");
                    break;
                }
                case 'S': {
                    lelDevice.curSlot = cmdParser.value;
                    break;
                }
                case 'P': {
                    lelDevice.setProgNum(cmdParser.value);
                    break;
                }
                case 'T': {
                    lelDevice.setTime(cmdParser.value);
                    break;
                }
                case 'A': {
                    lelDevice.setPwm(cmdParser.value);
                    break;
                }
                case 'R': {
                    lelDevice.curParam = cmdParser.value;
                    break;
                }
                case 'V': {
                    lelDevice.version = cmdParser.value;
                    tvVersion.setText(String.valueOf(lelDevice.version));
                    break;
                }
            }

            // Обновить таблицу
            try {
                for (int i = 0; i <= 3; i++)
                {
                    if (lelDevice.mode == Constants.PROCEDURE_MODE || lelDevice.mode == Constants.END_MODE)
                    {
                        slots.get(i + 1).setProgNum(lelDevice.slots.get(i).getProgNum());
                        slots.get(i + 1).setCurTime(lelDevice.slots.get(i).getCurTime());
                        slots.get(i + 1).setPwm(lelDevice.slots.get(i).getPwm());
                    } else
                    {
                        slots.get(i + 1).setProgNum(lelDevice.slots.get(i).getProgNum());
                        slots.get(i + 1).setPreTime(lelDevice.slots.get(i).getPreTime());
                        slots.get(i + 1).setPwm(lelDevice.slots.get(i).getPwm());
                    }
                    if ((lelDevice.mode == Constants.PROCEDURE_MODE ||
                         lelDevice.mode == Constants.SETTINGS_MODE ||
                         lelDevice.mode == Constants.END_MODE) && (lelDevice.curSlot == i))
                        slots.get(i + 1).setCurrent(true);
                    else
                        slots.get(i + 1).setCurrent(false);
                }
                // Обновить изображение на gvSlots
                slotAdapter.notifyDataSetChanged();
            } catch (Exception e) {
                Log.d(TAG, "ОШИБКА ПРЕОБРАЗОВАНИЯ ЧИСЛА... " + e.toString());
            }
        }
    }
}
