/*
Copyright (C) 2016, Vershinin Ivan
*/

package ru.vershinin.medical.lelmon;

import android.content.Context;
import android.graphics.Color;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.ImageSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import java.util.List;

import static ru.vershinin.medical.lelmon.R.drawable.tv_prog_brd;
import static ru.vershinin.medical.lelmon.R.drawable.tv_pwms_brd;
import static ru.vershinin.medical.lelmon.R.drawable.tv_sel_brd;
import static ru.vershinin.medical.lelmon.R.drawable.tv_time_brd;

public class SlotAdapter extends BaseAdapter {
    private Context context;
    private List<Slot> slots;
    private LelDevice device;
    private static int [] slotImages = {
            R.drawable.slot0,
            R.drawable.slot1,
            R.drawable.slot2,
            R.drawable.slot3,
            R.drawable.slot4 };
    private static LayoutInflater inflater = null;
    ImageSpan prog_is, time_is, pwms_is;
    SpannableString prog_text, time_text, pwms_text;

    public SlotAdapter(Context _context, List<Slot> _slots, LelDevice _device) {
        context = _context;
        inflater = (LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        slots = _slots;
        device = _device;
        // берем встроенную иконку share для создания объекта ImageSpan
        prog_is = new ImageSpan(context, R.drawable.progs);
        time_is = new ImageSpan(context, R.drawable.times);
        pwms_is = new ImageSpan(context, R.drawable.pwms);
        // создаем из строкового ресурса объект типа SpannableString
        prog_text = new SpannableString("_ ");
        time_text = new SpannableString("_ ");
        pwms_text = new SpannableString("_ ");
        // заменяем символы в строке с 0-позиции до 1 (т.е. один символ) иконкой, отсчет с 0
        prog_text.setSpan(prog_is, 0, 1, Spannable.SPAN_COMPOSING);
        time_text.setSpan(time_is, 0, 1, Spannable.SPAN_COMPOSING);
        pwms_text.setSpan(pwms_is, 0, 1, Spannable.SPAN_COMPOSING);
    }

    @Override
    public int getCount() {
        return slots.size();
    }

    @Override
    public Object getItem(int position) {
        return slots.get(position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    public class Holder
    {
        TextView tvNumProg;
        TextView tvTime;
        TextView tvPwm;
        ImageView img;
        ProgressBar pbSlotProgress;
    }

    @Override
    public View getView(final int position, View convertView, ViewGroup parent) {
        Holder holder;
        View rowView = convertView;

        if (convertView != null)
        {
            holder = (Holder)rowView.getTag();
        } else {
            holder = new Holder();
            rowView = inflater.inflate(R.layout.slot_list, null);
            holder.img = (ImageView) rowView.findViewById(R.id.ivSlotCap);
            holder.tvNumProg = (TextView) rowView.findViewById(R.id.tvNumProg);
            holder.tvTime = (TextView) rowView.findViewById(R.id.tvTime);
            holder.tvPwm = (TextView) rowView.findViewById(R.id.tvPwm);
            holder.pbSlotProgress = (ProgressBar) rowView.findViewById(R.id.pbSlotProgress);
            rowView.setTag(holder);
        }
        // Картинки
        holder.img.setImageResource(slotImages[position]);
        // Текст
        if (position == 0) {

            holder.tvNumProg.setText(prog_text);
            holder.tvTime.setText(time_text);
            holder.tvPwm.setText(pwms_text);

            switch (device.curParam) {
                case 0: {
                    holder.tvNumProg.setBackgroundResource(tv_sel_brd);
                    holder.tvTime.setBackgroundColor(Color.TRANSPARENT);
                    holder.tvPwm.setBackgroundColor(Color.TRANSPARENT);
                    break;
                }
                case 1: {
                    holder.tvNumProg.setBackgroundColor(Color.TRANSPARENT);
                    holder.tvTime.setBackgroundResource(tv_sel_brd);
                    holder.tvPwm.setBackgroundColor(Color.TRANSPARENT);
                    break;
                }
                case 2: {
                    holder.tvNumProg.setBackgroundColor(Color.TRANSPARENT);
                    holder.tvTime.setBackgroundColor(Color.TRANSPARENT);
                    holder.tvPwm.setBackgroundResource(tv_sel_brd);
                    break;
                }
                default: {
                    holder.tvNumProg.setBackgroundColor(Color.TRANSPARENT);
                    holder.tvTime.setBackgroundColor(Color.TRANSPARENT);
                    holder.tvPwm.setBackgroundColor(Color.TRANSPARENT);
                }
            }
            holder.pbSlotProgress.setVisibility(ProgressBar.INVISIBLE);
        } else {
            if (slots.get(position).getProgNum() >= 0)
                holder.tvNumProg.setText(String.format("%d", slots.get(position).getProgNum()));
            else
                holder.tvNumProg.setText("?");
            if (slots.get(position).getPreTime() >= 0)
                holder.tvTime.setText(String.format("%d/%d", slots.get(position).getPreTime(), slots.get(position).getCurTime()));
            else
                holder.tvTime.setText("?");
            if (slots.get(position).getPwm() >= 0)
                holder.tvPwm.setText(String.format("%d", slots.get(position).getPwm()));
            else
                holder.tvPwm.setText("?");
            if (slots.get(position).getCurrent()) {  // Цвет текущего слота
                holder.tvNumProg.setBackgroundResource(tv_sel_brd);
                holder.tvTime.setBackgroundResource(tv_sel_brd);
                holder.tvPwm.setBackgroundResource(tv_sel_brd);
            } else {  // Цвет не текущего слота
                holder.tvNumProg.setBackgroundResource(tv_prog_brd);
                holder.tvTime.setBackgroundResource(tv_time_brd);
                holder.tvPwm.setBackgroundResource(tv_pwms_brd);
            }
            if (device.mode == Constants.PROCEDURE_MODE && position == (device.curSlot + 1)) {
                holder.pbSlotProgress.setVisibility(ProgressBar.VISIBLE);
            } else
                holder.pbSlotProgress.setVisibility(ProgressBar.INVISIBLE);

        }

        rowView.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                if (position != 0)
                    Toast.makeText(context, "Этап: " + Integer.toString(position), Toast.LENGTH_SHORT).show();
            }
        });
        return rowView;
    }
}
